MODULE oce_trc
   !!======================================================================
   !!                      ***  MODULE  oce_trc  ***
   !! TOP :   variables shared between ocean and passive tracers
   !!======================================================================
   !! History :   1.0  !  2004-03  (C. Ethe)  original code
   !!             2.0  !  2007-12 (C. Ethe, G. Madec)  rewritting
   !!----------------------------------------------------------------------
#if defined key_top

#if defined key_crs

  !* Domain size *
   USE par_oce , ONLY :   jpi      =>   jpi        !: first  dimension of grid --> i 
   USE par_oce , ONLY :   jpj      =>   jpj        !: second dimension of grid --> j  
   USE par_oce , ONLY :   jpk      =>   jpk        !: number of levels  
   USE par_oce , ONLY :   jpim1    =>   jpim1      !: jpi - 1
   USE par_oce , ONLY :   jpjm1    =>   jpjm1      !: jpj - 1 
   USE par_oce , ONLY :   jpkm1    =>   jpkm1      !: jpk - 1  
   USE par_oce , ONLY :   jpij     =>   jpij       !: jpi x jpj
   USE par_oce , ONLY :   lk_esopa =>   lk_esopa   !: flag to activate the all option
   USE par_oce , ONLY :   jp_tem   =>   jp_tem     !: indice for temperature
   USE par_oce , ONLY :   jp_sal   =>   jp_sal     !: indice for salinity

   !* IO manager *
   USE in_out_manager

   !* Memory Allocation *
   USE wrk_nemo

   !* Timing *
   USE timing, ONLY : timing_start , timing_stop

   !* MPP library                         
   USE lib_mpp

   !* Fortran utilities                         
   USE lib_fortran_crs , ONLY : glob_sum => glob_sum_crs

   !* Lateral boundary conditions                         
   USE lbclnk

   !* physical constants *
   USE phycst

   !* 1D configuration
   USE c1d

   !* model domain *
   USE dom_oce , ONLY : narea => narea
   USE dom_oce , ONLY : nproc => nproc
   USE dom_oce , ONLY : nimpp => nimpp
   USE dom_oce , ONLY : njmpp => njmpp
   USE dom_oce , ONLY : nreci => nreci
   USE dom_oce , ONLY : nrecj => nrecj
   USE dom_oce , ONLY : nlci  => nlci
   USE dom_oce , ONLY : nldi  => nldi
   USE dom_oce , ONLY : nlei  => nlei
   USE dom_oce , ONLY : nlcj  => nlcj
   USE dom_oce , ONLY : nldj  => nldj
   USE dom_oce , ONLY : nlej  => nlej
   USE dom_oce , ONLY : nlcit  => nlcit
   USE dom_oce , ONLY : nldit  => nldit
   USE dom_oce , ONLY : nleit  => nleit
   USE dom_oce , ONLY : nlcjt  => nlcjt
   USE dom_oce , ONLY : nldjt  => nldjt
   USE dom_oce , ONLY : nlejt  => nlejt
   USE dom_oce , ONLY : nimppt => nimppt
   USE dom_oce , ONLY : njmppt => njmppt
   USE dom_oce , ONLY : ibonit => ibonit
   USE dom_oce , ONLY : ibonjt => ibonjt
   USE dom_oce , ONLY : lk_vvl => lk_vvl
   USE dom_oce , ONLY : rdt => rdt
   USE dom_oce , ONLY : ln_zco => ln_zco
   USE dom_oce , ONLY : ln_zps => ln_zps
   USE dom_oce , ONLY : ln_sco => ln_sco
   USE dom_oce , ONLY : neuler => neuler
   USE dom_oce , ONLY : mikt      => mikt          !: f-points (m)

   USE crs,  ONLY : mi0 => mi0 
   USE crs,  ONLY : mi1 => mi1 
   USE crs,  ONLY : mj0 => mj0 
   USE crs,  ONLY : mj1 => mj1 

   USE dom_oce , ONLY :  lzoom => lzoom 
   !USE dom_oce , ONLY :  => 

   !* horizontal mesh *
   USE crs , ONLY :   glamt      =>   glamt_crs      !: longitude of t-point (degre)  
   USE crs , ONLY :   glamu      =>   glamu_crs      !: longitude of t-point (degre)  
   USE crs , ONLY :   glamv      =>   glamv_crs      !: longitude of t-point (degre)  
   USE crs , ONLY :   glamf      =>   glamf_crs      !: longitude of t-point (degre)  
   USE crs , ONLY :   gphit      =>   gphit_crs      !: latitude  of t-point (degre)   
   USE crs , ONLY :   gphiu      =>   gphiu_crs      !: latitude  of t-point (degre)   
   USE crs , ONLY :   gphiv      =>   gphiv_crs      !: latitude  of t-point (degre)   
   USE crs , ONLY :   gphif      =>   gphif_crs      !: latitude  of t-point (degre)   
   USE crs , ONLY :   e1t        =>   e1t_crs        !: horizontal scale factors at t-point (m)  
   USE crs , ONLY :   e2t        =>   e2t_crs        !: horizontal scale factors at t-point (m)   
   USE crs , ONLY :   e1e2t      =>   e1e2t_crs      !: cell surface at t-point (m2)
   USE crs , ONLY :   e1u        =>   e1u_crs        !: horizontal scale factors at u-point (m)
   USE crs , ONLY :   e2u        =>   e2u_crs        !: horizontal scale factors at u-point (m)
   USE crs , ONLY :   e1v        =>   e1v_crs        !: horizontal scale factors at v-point (m)
   USE crs , ONLY :   e2v        =>   e2v_crs        !: horizontal scale factors at v-point (m)  

#if defined key_vvl 
   USE crs , ONLY :   e3t        =>  e3t_n_crs         !: vertical scale factors at t-
   USE crs , ONLY :   e3u        =>  e3u_n_crs         !: vertical scale factors at u-
   USE crs , ONLY :   e3v        =>  e3v_n_crs         !: vertical scale factors v-
   USE crs , ONLY :   e3w        =>  e3w_n_crs         !: w-points (m)
   USE crs , ONLY :   e3t_n      =>  e3t_n_crs         !: vertical scale factors at t-
   USE crs , ONLY :   e3u_n      =>  e3u_n_crs         !: vertical scale factors at u-
   USE crs , ONLY :   e3v_n      =>  e3v_n_crs         !: vertical scale factors v-
   USE crs , ONLY :   e3w_n      =>  e3w_n_crs         !: w-points (m)
   USE crs , ONLY :   e3t_a      =>  e3t_a_crs         !: vertical scale factors at t-
   USE crs , ONLY :   e3u_a      =>  e3u_a_crs         !: vertical scale factors at u-
   USE crs , ONLY :   e3v_a      =>  e3v_a_crs         !: vertical scale factors v-
   USE crs , ONLY :   e3w_a      =>  e3w_a_crs         !: w-points (m)
   USE crs , ONLY :   fse3t      =>  e3t_n_crs         !: vertical scale factors at t-
   USE crs , ONLY :   fse3u      =>  e3u_n_crs         !: vertical scale factors at u-
   USE crs , ONLY :   fse3v      =>  e3v_n_crs         !: vertical scale factors v-
   USE crs , ONLY :   fse3w      =>  e3w_n_crs         !: w-points (m)
   USE crs , ONLY :   gdept      =>  gdept_n_crs       !: depth of t-points (m)
   USE crs , ONLY :   gdept_crs  =>  gdept_n_crs       !: depth of t-points (m)
   USE crs , ONLY :   gdept_n    =>  gdept_n_crs       !: depth of t-points (m)
   USE crs , ONLY :   fse3t_b    =>  e3t_b_crs         !: vertical scale factors at t-
   USE crs , ONLY :   fse3t_n    =>  e3t_n_crs         !: vertical scale factors at t-
   USE crs , ONLY :   fse3t_a    =>  e3t_a_crs         !: vertical scale factors at t-
   USE crs , ONLY :   fsdept_n   =>  gdept_n_crs       !: depth of t-points (m)
   USE crs , ONLY :   e3t_max_crs => e3t_max_n_crs
   USE crs , ONLY :   e3u_max_crs => e3u_max_n_crs
   USE crs , ONLY :   e3v_max_crs => e3v_max_n_crs
   USE crs , ONLY :   e3w_max_crs => e3w_max_n_crs
#else
   USE crs , ONLY :   e3t        =>  e3t_0_crs         !: vertical scale factors at t-
   USE crs , ONLY :   e3u        =>  e3u_0_crs         !: vertical scale factors at u-
   USE crs , ONLY :   e3v        =>  e3v_0_crs         !: vertical scale factors v-
   USE crs , ONLY :   e3w        =>  e3w_0_crs         !: w-points (m)
   USE crs , ONLY :   e3t_n      =>  e3t_0_crs         !: vertical scale factors at t-
   USE crs , ONLY :   e3u_n      =>  e3u_0_crs         !: vertical scale factors at u-
   USE crs , ONLY :   e3v_n      =>  e3v_0_crs         !: vertical scale factors v-
   USE crs , ONLY :   e3w_n      =>  e3w_0_crs         !: w-points (m)
   USE crs , ONLY :   e3t_a      =>  e3t_0_crs         !: vertical scale factors at t-
   USE crs , ONLY :   e3u_a      =>  e3u_0_crs         !: vertical scale factors at u-
   USE crs , ONLY :   e3v_a      =>  e3v_0_crs         !: vertical scale factors v-
   USE crs , ONLY :   e3w_a      =>  e3w_0_crs         !: w-points (m)
   USE crs , ONLY :   fse3t      =>  e3t_0_crs         !: vertical scale factors at t-
   USE crs , ONLY :   fse3u      =>  e3u_0_crs         !: vertical scale factors at u-
   USE crs , ONLY :   fse3v      =>  e3v_0_crs         !: vertical scale factors v-
   USE crs , ONLY :   fse3w      =>  e3w_0_crs         !: w-points (m)
   USE crs , ONLY :   gdept      =>  gdept_0_crs       !: depth of t-points (m)
   USE crs , ONLY :   gdepw      =>  gdepw_0_crs       !: depth of t-points (m)
   USE crs , ONLY :   gdept_crs  =>  gdept_0_crs       !: depth of t-points (m)
   USE crs , ONLY :   gdepw_crs  =>  gdepw_0_crs       !: depth of t-points (m)
   USE crs , ONLY :   gdept_n    =>  gdept_0_crs       !: depth of t-points (m)
   USE crs , ONLY :   fse3t_b    =>  e3t_0_crs         !: vertical scale factors at t-
   USE crs , ONLY :   fse3t_n    =>  e3t_0_crs         !: vertical scale factors at t-
   USE crs , ONLY :   fse3t_a    =>  e3t_0_crs         !: vertical scale factors at t-
   USE crs , ONLY :   fsdept_n   =>  gdept_0_crs       !: depth of t-points (m)
   USE crs , ONLY :   e3t_max_crs => e3t_max_0_crs
   USE crs , ONLY :   e3u_max_crs => e3u_max_0_crs
   USE crs , ONLY :   e3v_max_crs => e3v_max_0_crs
   USE crs , ONLY :   e3w_max_crs => e3w_max_0_crs
#endif
   USE crs , ONLY :   e3t_0        =>  e3t_0_crs         !: vertical scale factors at t-
   USE crs , ONLY :   e3u_0        =>  e3u_0_crs         !: vertical scale factors at t-
   USE crs , ONLY :   e3v_0        =>  e3v_0_crs         !: vertical scale factors at t-
   USE crs , ONLY :   e3w_0        =>  e3w_0_crs         !: vertical scale factors at t-

   USE crs , ONLY :   ff         =>  ff_crs         !: f-points (m)
   USE crs , ONLY :   gdept_0    =>  gdept_0_crs       !: depth of t-points (m)
   USE dom_oce , ONLY :   gdept_1d   =>  gdept_1d      !: depth of t-points (m)
#if defined key_zco
   USE crs , ONLY :   gdept      =>  gdept_0_crs       !: depth of t-points (m)
   USE crs , ONLY :   gdepw      =>  gdepw_crs       !: depth of t-points (m)
#endif
  !* masks, bathymetry *
   USE crs , ONLY :   mbkt       =>   mbkt_crs       !: vertical index of the bottom last T- ocean level
   USE crs , ONLY :   mbku       =>   mbku_crs       !: vertical index of the bottom last U- ocean level
   USE crs , ONLY :   mbkv       =>   mbkv_crs       !: vertical index of the bottom last V- ocean level
   USE crs , ONLY :   tmask_i    =>   tmask_i_crs    !: Interior mask at t-points
   USE crs , ONLY :   tmask      =>   tmask_crs      !: land/ocean mask at t-points
   USE crs , ONLY :   umask      =>   umask_crs      !: land/ocean mask at u-points   
   USE crs , ONLY :   vmask      =>   vmask_crs      !: land/ocean mask at v-points 
   USE crs , ONLY :   fmask      =>   fmask_crs      !: land/ocean mask at f-points 

 !* ocean fields: here now and after fields *
!cbr?   USE crs , ONLY :   ua      =>    ua_crs      !: i-horizontal velocity (m s-1) 
!cbr?   USE crs , ONLY :   va      =>    va_crs      !: j-horizontal velocity (m s-1)
   USE crs , ONLY :   un      =>    un_crs      !: i-horizontal velocity (m s-1) 
   USE crs , ONLY :   vn      =>    vn_crs      !: j-horizontal velocity (m s-1)
   USE crs , ONLY :   wn      =>    wn_crs      !: vertical velocity (m s-1)  
   USE crs , ONLY :   tsn     =>    tsn_crs     !: 4D array contaning ( tn, sn )
   USE crs , ONLY :   tsb     =>    tsb_crs     !: 4D array contaning ( tb, sb )
   USE crs , ONLY :   tsa     =>    tsa_crs     !: 4D array contaning ( ta, sa )
   USE crs , ONLY :   rhop    =>    rhop_crs    !: potential volumic mass (kg m-3) 
   USE crs , ONLY :   rhd     =>    rhd_crs    !: in situ density anomalie rhd=(rho-rau0)/rau0 (no units)
   USE crs , ONLY :   rn2b    =>    rb2_crs     !: in situ density anomalie rhd=(rho-rau0)/rau0 (no units)
   USE crs , ONLY :   rab_n   =>    rab_crs_n     !: in situ density anomalie rhd=(rho-rau0)/rau0 (no units)
   USE crs , ONLY :   hdivn   =>    hdivn_crs   !: horizontal divergence (1/s)
   USE crs , ONLY :   hdivb   =>    hdivb_crs   !: horizontal divergence (1/s)
   USE crs , ONLY :   sshb    =>    sshb_crs    !: sea surface height at t-point [m]   
   USE crs , ONLY :   sshn    =>    sshn_crs    !: sea surface height at t-point [m]   
   USE crs , ONLY :   ssha    =>    ssha_crs    !: sea surface height at t-point [m]   

   !* surface fluxes *
   USE crs , ONLY :   utau       =>    utau_crs       !: i-surface stress component
   USE crs , ONLY :   vtau       =>    vtau_crs       !: j-surface stress component
   USE crs , ONLY :   wndm       =>    wndm_crs       !: 10m wind speed 
   USE crs , ONLY :   qsr        =>    qsr_crs        !: penetrative solar radiation (w m-2)  
   USE crs , ONLY :   emp        =>    emp_crs        !: freshwater budget: volume flux               [Kg/m2/s]
   USE crs , ONLY :   emp_b      =>    emp_b_crs      !: freshwater budget: volume flux               [Kg/m2/s]
   USE crs , ONLY :   sfx        =>    sfx_crs        !: freshwater budget: concentration/dillution   [Kg/m2/s]
   USE crs , ONLY :   fmmflx     =>    fmmflx_crs     !: freshwater budget: volume flux               [Kg/m2/s]
   USE crs , ONLY :   rnf        =>    rnf_crs        !: river runoff   [Kg/m2/s]
   USE crs , ONLY :   fr_i       =>    fr_i_crs       !: ice fraction (between 0 to 1)
   USE trcnam_trp , ONLY :  aht0     =>   rn_ahtrc_0        !: horizontal eddy diffusivity for tracers (m2/s)
   USE crs , ONLY :  ahtu     =>   ahtu_crs        !: lateral diffusivity coef. at u-points 
   USE crs , ONLY :  ahtv     =>   ahtv_crs        !: lateral diffusivity coef. at v-points 
   USE crs , ONLY :  ahtw     =>   ahtw_crs        !: lateral diffusivity coef. at w-points 
   USE crs , ONLY :  ahtt     =>   ahtt_crs        !: lateral diffusivity coef. at t-points
   USE crs , ONLY :  r_fact_lap     =>  r_fact_lap_crs        !: enhanced zonal diffusivity coefficient
   USE ldftra_oce , ONLY :  rldf     =>   rldf
   USE crs , ONLY :  trc_i => trc_i_crs
   USE crs , ONLY :  trc_o => trc_o_crs
   USE crs , ONLY :   avt        =>   avt_crs         !: vert. diffusivity coef. at w-point for temp  
#if defined key_zdfddm
   USE crs , ONLY :   avs        =>   avs_crs         !: salinity vertical diffusivity coeff. at w-point
#endif

!cbr   USE trc_oce
   USE trc_oce, ONLY : lk_offline
   USE trc_oce, ONLY : nn_dttrc

   USE crs , ONLY :   nmln        =>   nmln_crs        !: number of level in the mixed layer
   USE crs , ONLY :   hmld        =>   hmld_crs        !: mixing layer depth (turbocline)
   USE crs , ONLY :   hmlp        =>   hmlp_crs        !: mixed layer depth  (rho=rho0+zdcrit) (m)
   USE crs , ONLY :   hmlpt       =>   hmlpt_crs       !: mixed layer depth at t-points (m)

  !* direction of lateral diffusion *
#if   defined key_ldfslp
   USE crs , ONLY :   uslp       =>   uslp_crs         !: i-direction slope at u-, w-points
   USE crs , ONLY :   vslp       =>   vslp_crs         !: j-direction slope at v-, w-points
   USE crs , ONLY :   wslpi      =>   wslpi_crs        !: i-direction slope at u-, w-points
   USE crs , ONLY :   wslpj      =>   wslpj_crs        !: j-direction slope at v-, w-points
#endif

#else

   !!----------------------------------------------------------------------
   !!   'key_top'                                                TOP models
   !!----------------------------------------------------------------------

   !* Domain size *
   USE par_oce , ONLY :   jpi      =>   jpi        !: first  dimension of grid --> i 
   USE par_oce , ONLY :   jpj      =>   jpj        !: second dimension of grid --> j  
   USE par_oce , ONLY :   jpk      =>   jpk        !: number of levels  
   USE par_oce , ONLY :   jpim1    =>   jpim1      !: jpi - 1
   USE par_oce , ONLY :   jpjm1    =>   jpjm1      !: jpj - 1 
   USE par_oce , ONLY :   jpkm1    =>   jpkm1      !: jpk - 1  
   USE par_oce , ONLY :   jpij     =>   jpij       !: jpi x jpj
   USE par_oce , ONLY :   lk_esopa =>   lk_esopa   !: flag to activate the all option
   USE par_oce , ONLY :   jp_tem   =>   jp_tem     !: indice for temperature
   USE par_oce , ONLY :   jp_sal   =>   jp_sal     !: indice for salinity

  !* model domain *
   USE dom_oce , ONLY : narea => narea
   USE dom_oce , ONLY : nproc => nproc
   USE dom_oce , ONLY : nimpp => nimpp
   USE dom_oce , ONLY : njmpp => njmpp
   USE dom_oce , ONLY : nreci => nreci
   USE dom_oce , ONLY : nrecj => nrecj
   USE dom_oce , ONLY : nlci  => nlci
   USE dom_oce , ONLY : nldi  => nldi
   USE dom_oce , ONLY : nlei  => nlei
   USE dom_oce , ONLY : nlcj  => nlcj
   USE dom_oce , ONLY : nldj  => nldj
   USE dom_oce , ONLY : nlej  => nlej
   USE dom_oce , ONLY : nlcit  => nlcit
   USE dom_oce , ONLY : nldit  => nldit
   USE dom_oce , ONLY : nleit  => nleit
   USE dom_oce , ONLY : nlcjt  => nlcjt
   USE dom_oce , ONLY : nldjt  => nldjt
   USE dom_oce , ONLY : nlejt  => nlejt
   USE dom_oce , ONLY : nimppt => nimppt
   USE dom_oce , ONLY : njmppt => njmppt
   USE dom_oce , ONLY : ibonit => ibonit
   USE dom_oce , ONLY : ibonjt => ibonjt
   USE dom_oce , ONLY : lk_vvl => lk_vvl
   USE dom_oce , ONLY : rdt => rdt
   USE dom_oce , ONLY : ln_zco => ln_zco
   USE dom_oce , ONLY : ln_zps => ln_zps
   USE dom_oce , ONLY : ln_sco => ln_sco
   USE dom_oce , ONLY : neuler => neuler

   USE dom_oce,  ONLY : mi0 => mi0
   USE dom_oce,  ONLY : mi1 => mi1
   USE dom_oce,  ONLY : mj0 => mj0
   USE dom_oce,  ONLY : mj1 => mj1

   USE dom_oce , ONLY :   glamt      =>   glamt      !: longitude of t-point (degre)  
   USE dom_oce , ONLY :   glamu      =>   glamu      !: longitude of t-point (degre)  
   USE dom_oce , ONLY :   glamv      =>   glamv      !: longitude of t-point (degre)  
   USE dom_oce , ONLY :   glamf      =>   glamf      !: longitude of t-point (degre)  
   USE dom_oce , ONLY :   gphit      =>   gphit      !: latitude  of t-point (degre)   
   USE dom_oce , ONLY :   gphiu      =>   gphiu      !: latitude  of t-point (degre)   
   USE dom_oce , ONLY :   gphiv      =>   gphiv      !: latitude  of t-point (degre)   
   USE dom_oce , ONLY :   gphif      =>   gphif     !: latitude  of t-point (degre)   
   USE dom_oce , ONLY :   e1t        =>   e1t        !: horizontal scale factors at t-point (m)  
   USE dom_oce , ONLY :   e2t        =>   e2t        !: horizontal scale factors at t-point (m)   
   USE dom_oce , ONLY :   e1e2t      =>   e1e2t      !: cell surface at t-point (m2)
   USE dom_oce , ONLY :   e1u        =>   e1u        !: horizontal scale factors at u-point (m)
   USE dom_oce , ONLY :   e2u        =>   e2u        !: horizontal scale factors at u-point (m)
   USE dom_oce , ONLY :   e1v        =>   e1v        !: horizontal scale factors at v-point (m)
   USE dom_oce , ONLY :   e2v        =>   e2v        !: horizontal scale factors at v-point (m)  
   USE dom_oce , ONLY :   e3t        =>  e3t_0         !: vertical scale factors at t-
   USE dom_oce , ONLY :   e3t_0      =>  e3t_0         !: vertical scale factors at t-
#if defined key_vvl 
   USE dom_oce , ONLY :   fse3t_b    =>  e3t_b
   USE dom_oce , ONLY :   fse3t_n    =>  e3t_n
   USE dom_oce , ONLY :   fse3t      =>  e3t_n
   USE dom_oce , ONLY :   fse3u      =>  e3u_n
   USE dom_oce , ONLY :   fse3v      =>  e3v_n
   USE dom_oce , ONLY :   fse3w      =>  e3w_n
   USE dom_oce , ONLY :   fse3t_a    =>  e3t_a
   USE dom_oce , ONLY :     e3t_b    =>  e3t_b
   USE dom_oce , ONLY :     e3t_n    =>  e3t_n
   USE dom_oce , ONLY :     e3t_a    =>  e3t_a
   USE dom_oce , ONLY :     e3u_n    =>  e3u_n
   USE dom_oce , ONLY :     e3v_n    =>  e3v_n
   USE dom_oce , ONLY :   e3u        =>  e3u_n         !: vertical scale factors at u-
   USE dom_oce , ONLY :   e3u_0      =>  e3u_0         !: vertical scale factors at u-
   USE dom_oce , ONLY :   e3v        =>  e3v_n         !: vertical scale factors v-
   USE dom_oce , ONLY :   e3v_0      =>  e3v_0         !: vertical scale factors v-
   USE dom_oce , ONLY :   e3w_n      =>  e3w_n         !: w-points (m)
   USE dom_oce , ONLY :   e3w        =>  e3w_n         !: w-points (m)
   USE dom_oce , ONLY :   e3w_0      =>  e3w_0         !: w-points (m)
   USE dom_oce , ONLY :   e3f        =>  e3f_n         !: f-points (m)
   USE dom_oce , ONLY :   gdept_n    =>  gdept_n         !: f-points (m)
   USE dom_oce , ONLY :  fsdept_n    =>  gdept_n         !: f-points (m)
#else
   USE dom_oce , ONLY :   fse3t_n    =>  e3t_0
   USE dom_oce , ONLY :   fse3t      =>  e3t_0
   USE dom_oce , ONLY :   fse3u      =>  e3u_0
   USE dom_oce , ONLY :   fse3v      =>  e3v_0
   USE dom_oce , ONLY :   fse3w      =>  e3w_0
   USE dom_oce , ONLY :   fse3t_b    =>  e3t_0
   USE dom_oce , ONLY :   fse3t_a    =>  e3t_0
   USE dom_oce , ONLY :     e3t_a    =>  e3t_0
   USE dom_oce , ONLY :   e3u        =>  e3u_0         !: vertical scale factors at u-
   USE dom_oce , ONLY :   e3u_0      =>  e3u_0         !: vertical scale factors at u-
   USE dom_oce , ONLY :   e3v        =>  e3v_0         !: vertical scale factors v-
   USE dom_oce , ONLY :   e3v_0      =>  e3v_0         !: vertical scale factors v-
   USE dom_oce , ONLY :   e3w        =>  e3w_0         !: w-points (m)
   USE dom_oce , ONLY :   e3w_0      =>  e3w_0         !: w-points (m)
   USE dom_oce , ONLY :   e3f        =>  e3f_0         !: f-points (m)
   USE dom_oce , ONLY :   gdept_n    =>  gdept_0         !: f-points (m)
   USE dom_oce , ONLY :  fsdept_n    =>  gdept_0         !: f-points (m)
#endif
   USE dom_oce , ONLY :   ff         =>  ff         !: f-points (m)
   USE dom_oce , ONLY :   gdept_0    =>  gdept_0         !: f-points (m)
   USE dom_oce , ONLY :   gdept_1d   => gdept_1d          !: f-points (m)
   USE dom_oce , ONLY :   tmask      => tmask          !: f-points (m)
   USE dom_oce , ONLY :   umask      => umask          !: f-points (m)
   USE dom_oce , ONLY :   vmask      => vmask          !: f-points (m)
   USE dom_oce , ONLY :   tmask_i      => tmask_i          !: f-points (m)
   USE dom_oce , ONLY :   mbkt      => mbkt          !: f-points (m)
   USE dom_oce , ONLY :   mbku      => mbku          !: f-points (m)
   USE dom_oce , ONLY :   mbkv      => mbkv          !: f-points (m)
   USE dom_oce , ONLY :   mikt      => mikt          !: f-points (m)

   !* IO manager *
   USE in_out_manager    
 
   !* Memory Allocation *
   USE wrk_nemo      
 
   !* Timing *
   USE timing    
 
   !* MPP library                         
   USE lib_mpp 

   !* Fortran utilities                         
   USE lib_fortran , ONLY : glob_sum => glob_sum

   !* Lateral boundary conditions                         
   USE lbclnk

   !* physical constants *
   USE phycst            

   !* 1D configuration
   USE c1d                                         

   !* model domain *
   !cbr USE dom_oce , ONLY : e3w_0
   USE dom_oce , ONLY :  lzoom => lzoom 

   USE domvvl, ONLY : un_td, vn_td          !: thickness diffusion transport
   USE domvvl, ONLY : ln_vvl_ztilde         !: ztilde vertical coordinate
   USE domvvl, ONLY : ln_vvl_layer          !: level  vertical coordinate

   !* ocean fields: here now and after fields *
   USE oce , ONLY :   ua      =>    ua      !: i-horizontal velocity (m s-1) 
   USE oce , ONLY :   va      =>    va      !: j-horizontal velocity (m s-1)
   USE oce , ONLY :   un      =>    un      !: i-horizontal velocity (m s-1) 
   USE oce , ONLY :   vn      =>    vn      !: j-horizontal velocity (m s-1)
   USE oce , ONLY :   wn      =>    wn      !: vertical velocity (m s-1)  
   USE oce , ONLY :   tsn     =>    tsn     !: 4D array contaning ( tn, sn )
   USE oce , ONLY :   tsb     =>    tsb     !: 4D array contaning ( tb, sb )
   USE oce , ONLY :   tsa     =>    tsa     !: 4D array contaning ( ta, sa )
   USE oce , ONLY :   rhop    =>    rhop    !: potential volumic mass (kg m-3) 
   USE oce , ONLY :   rhd     =>    rhd     !: in situ density anomalie rhd=(rho-rau0)/rau0 (no units)
   USE oce , ONLY :   rab_n     =>    rab_n   !: in situ density anomalie rhd=(rho-rau0)/rau0 (no units)
   USE oce , ONLY :   rn2b    =>    rn2b    !: in situ density anomalie rhd=(rho-rau0)/rau0 (no units)
   USE oce , ONLY :   hdivn   =>    hdivn   !: horizontal divergence (1/s)
   USE oce , ONLY :   rotn    =>    rotn    !: relative vorticity    [s-1]
   USE oce , ONLY :   hdivb   =>    hdivb   !: horizontal divergence (1/s)
   USE oce , ONLY :   rotb    =>    rotb    !: relative vorticity    [s-1]
   USE oce , ONLY :   sshn    =>    sshn    !: sea surface height at t-point [m]   
   USE oce , ONLY :   sshb    =>    sshb    !: sea surface height at t-point [m]   
   USE oce , ONLY :   ssha    =>    ssha    !: sea surface height at t-point [m]   
   USE oce , ONLY :   l_traldf_rot => l_traldf_rot  !: rotated laplacian operator for lateral diffusion

   !* surface fluxes *
   USE sbc_oce , ONLY :   utau       =>    utau       !: i-surface stress component
   USE sbc_oce , ONLY :   vtau       =>    vtau       !: j-surface stress component
   USE sbc_oce , ONLY :   wndm       =>    wndm       !: 10m wind speed 
   USE sbc_oce , ONLY :   qsr        =>    qsr        !: penetrative solar radiation (w m-2)
   USE sbc_oce , ONLY :   emp        =>    emp        !: freshwater budget: volume flux               [Kg/m2/s]
   USE sbc_oce , ONLY :   emp_b      =>    emp_b      !: freshwater budget: volume flux               [Kg/m2/s]
   USE sbc_oce , ONLY :   fmmflx     =>    fmmflx     !: freshwater budget: volume flux               [Kg/m2/s]
   USE sbc_oce , ONLY :   rnf        =>    rnf        !: river runoff   [Kg/m2/s]
   USE sbc_oce , ONLY :   ln_dm2dc   =>    ln_dm2dc   !: Diurnal Cycle 
   USE sbc_oce , ONLY :   fr_i       =>    fr_i       !: ice fraction (between 0 to 1)
   USE traqsr  , ONLY :   rn_abs     =>    rn_abs     !: fraction absorbed in the very near surface
   USE traqsr  , ONLY :   rn_si0     =>    rn_si0     !: very near surface depth of extinction
   USE traqsr  , ONLY :   ln_qsr_bio =>    ln_qsr_bio !: flag to use or not the biological fluxes for light
   USE sbcrnf  , ONLY :   rnfmsk     =>    rnfmsk     !: mixed adv scheme in runoffs vicinity (hori.) 
   USE sbcrnf  , ONLY :   rnfmsk_z   =>    rnfmsk_z   !: mixed adv scheme in runoffs vicinity (vert.)
   USE sbcrnf  , ONLY :   h_rnf      =>    h_rnf      !: river runoff   [Kg/m2/s]
   USE sbcrnf  , ONLY :   nk_rnf     =>    nk_rnf     !: depth of runoff in model level
   USE trc , ONLY :  trc_i => trc_i
   USE trc , ONLY :  trc_o => trc_o

   USE trc_oce

   !* lateral diffusivity (tracers) *
   USE ldftra_oce , ONLY :  rldf     =>   rldf        !: multiplicative coef. for lateral diffusivity
   USE ldftra_oce , ONLY :  rn_aht_0 =>   rn_aht_0    !: horizontal eddy diffusivity for tracers (m2/s)
   USE ldftra_oce , ONLY :  aht0     =>   aht0        !: horizontal eddy diffusivity for tracers (m2/s)
   USE ldftra_oce , ONLY :  ahtb0    =>   ahtb0       !: background eddy diffusivity for isopycnal diff. (m2/s)
   USE ldftra_oce , ONLY :  ahtu     =>   ahtu        !: lateral diffusivity coef. at u-points 
   USE ldftra_oce , ONLY :  ahtv     =>   ahtv        !: lateral diffusivity coef. at v-points 
   USE ldftra_oce , ONLY :  ahtw     =>   ahtw        !: lateral diffusivity coef. at w-points 
   USE ldftra_oce , ONLY :  ahtt     =>   ahtt        !: lateral diffusivity coef. at t-points
   USE ldftra_oce , ONLY :  aeiv0    =>   aeiv0       !: eddy induced velocity coefficient (m2/s) 
   USE ldftra_oce , ONLY :  aeiu     =>   aeiu        !: eddy induced velocity coef. at u-points (m2/s)   
   USE ldftra_oce , ONLY :  aeiv     =>   aeiv        !: eddy induced velocity coef. at v-points (m2/s) 
   USE ldftra_oce , ONLY :  aeiw     =>   aeiw        !: eddy induced velocity coef. at w-points (m2/s) 
   USE ldftra_oce , ONLY :  lk_traldf_eiv  =>  lk_traldf_eiv     !: eddy induced velocity flag
   USE ldftra_oce , ONLY :  r_fact_lap     =>  r_fact_lap        !: enhanced zonal diffusivity coefficient

   !* vertical diffusion *
   USE zdf_oce , ONLY :   avt        =>   avt         !: vert. diffusivity coef. at w-point for temp  
# if defined key_zdfddm
   USE zdfddm  , ONLY :   avs        =>   avs         !: salinity vertical diffusivity coeff. at w-point
# endif

   !* mixing & mixed layer depth *
   USE zdfmxl , ONLY :   nmln        =>   nmln        !: number of level in the mixed layer
   USE zdfmxl , ONLY :   hmld        =>   hmld        !: mixing layer depth (turbocline)
   USE zdfmxl , ONLY :   hmlp        =>   hmlp        !: mixed layer depth  (rho=rho0+zdcrit) (m)
   USE zdfmxl , ONLY :   hmlpt       =>   hmlpt       !: mixed layer depth at t-points (m)

   !* direction of lateral diffusion *
   USE ldfslp , ONLY :   lk_ldfslp  =>  lk_ldfslp     !: slopes flag
# if   defined key_ldfslp
   USE ldfslp , ONLY :   uslp       =>   uslp         !: i-direction slope at u-, w-points
   USE ldfslp , ONLY :   vslp       =>   vslp         !: j-direction slope at v-, w-points
   USE ldfslp , ONLY :   wslpi      =>   wslpi        !: i-direction slope at u-, w-points
   USE ldfslp , ONLY :   wslpj      =>   wslpj        !: j-direction slope at v-, w-points
# endif

   USE diaar5 , ONLY :   lk_diaar5  =>   lk_diaar5

#endif


   USE dom_oce , ONLY : ndastp
   USE sbc_oce , ONLY : nn_ice_embd
   USE sbc_oce , ONLY : ln_cpl
   USE sbc_oce , ONLY : ln_rnf
   USE sbc_oce , ONLY : ncpl_qsr_freq

#else
   !!----------------------------------------------------------------------
   !!  Empty module :                                     No passive tracer
   !!----------------------------------------------------------------------
#endif

   !!----------------------------------------------------------------------
   !! NEMO/TOP 3.3 , NEMO Consortium (2010)
   !! $Id$
   !! Software governed by the CeCILL licence (NEMOGCM/NEMO_CeCILL.txt)
   !!======================================================================
END MODULE oce_trc
